function TranslateAssistant() {
}

TranslateAssistant.prototype.setup = function() {
	this.setupLanguages();
	this.textModel = {
		resultDisabled: true,
		text: "",
		result: ""
	};

	this.controller.setupWidget("text", {
		multiline: true,
		modelProperty: "text",
		enterSubmits: true,
		changeOnKeyPress: true
	}, this.textModel);
	this.controller.setupWidget("result", {
		multiline: true,
		modelProperty: "result",
		disabledProperty: "resultDisabled"
	}, this.textModel);

	var pref = cookie.get();
	if (pref) {
		this.languageModel = {
			from: pref.from,
			to: pref.to
		};
	} else {
		this.languageModel = {
			from: "en",
			to: "de"
		};
	}

	this.controller.setupWidget("fromLanguageSelector", {
		choices: this.languages,
		modelProperty: "from"
	}, this.languageModel);
	this.controller.setupWidget("toLanguageSelector", {
		choices: this.languages,
		modelProperty: "to"
	}, this.languageModel);

	this.controller.setupWidget("translate", {
		type: Mojo.Widget.activityButton
	}, this.translateButtonModel = {
		buttonLabel: "Translate",
		disabled: true
	});

	this.controller.setupWidget(Mojo.Menu.appMenu, { omitDefaultItems: true}, {
		visible: true,
		items: [
			{ label: "Paste", command: "paste"},
			{ label: "Copy", command: "copy"},
			{ label: "About Translator", command: "about" }
		]
	});

	this.controller.listen("translate", Mojo.Event.tap, this.doTranslate.bindAsEventListener(this));
	this.controller.listen("switch", Mojo.Event.tap, this.switchLanguages.bindAsEventListener(this));

	this.controller.listen("sendText", Mojo.Event.tap, this.sendText.bindAsEventListener(this));
	this.controller.listen("clearInput", Mojo.Event.tap, this.clearInput.bindAsEventListener(this));

	// we don't use a command button instead of a regular button, (yet)
	/*	var commandButtons = {items: [{
	 },{
	 label: "Translate",
	 command: "do-Translate",
	 width: 250
	 },{
	 }]};
	 this.controller.setupWidget(Mojo.Menu.commandMenu, undefined, commandButtons);
	 */
	this.controller.listen("text", Mojo.Event.propertyChange, this.textFieldChanged.bindAsEventListener(this));
}

TranslateAssistant.prototype.textFieldChanged = function(event) {
	this.translateButtonModel.disabled = this.textModel.text.length == 0;
	this.controller.modelChanged(this.translateButtonModel, this);
	// submit on enter
	if (event && Mojo.Char.isEnterKey(event.originalEvent.keyCode)) {
		if (!this.translateButtonModel.disabled) {
			this.doTranslate();
			Event.stop(event);
		}
	}
}

TranslateAssistant.prototype.handleCommand = function(event) {
	if (event.type == Mojo.Event.command) {
		switch (event.command) {
			case "do-Translate": // not used at the moment
				this.doTranslate(event);
				break;
			case "paste":
				if (PalmSystem && PalmSystem.paste) {
					$("text").mojo.focus();
					PalmSystem.paste();
				}
				break;
			case "copy":
				var document = this.controller.stageController.window.document;
			//		var e = document.createElement("div");
			//		document.body.insertBefore(e);
			//		e.innerText = 'Ich bin neu hier!';
			//		r.add(e);
			//		r.execCommand("copy");
				$("hiddenResult").select();
				document.execCommand("copy");
				break;
			case "about":
				var title = "<div align='center'>" + Mojo.Controller.appInfo.title +
							" - v" + Mojo.Controller.appInfo.version + "</div>";
				var message = "<div align='center'>Copyright 2009, Janni Kovacs<br/><a href='http://www.u-mass.de'>u-mass.de</a></div>";
				this.controller.showAlertDialog({
					onChoose: function(value) {
					},
					title: title,
					message: message,
					choices:[{label: "OK", value:""}]
				});
				break;
		}
	}
}

TranslateAssistant.prototype.setupLanguages = function() {

	this.languages = [
		{ label: "Albanian", value: "sq"},
//		{ label: "Arabic", value: "ar"},
		{ label: "Bulgarian", value: "bg"},
		{ label: "Catalan", value: "ca"},
		{ label: "Chinese", value: "zh"},
		{ label: "Chinese simplified", value: "zh-CN"},
		{ label: "Chinese traditional", value: "zh-TW"},
		{ label: "Croatian", value: "hr"},
		{ label: "Czech", value: "cs"},
		{ label: "Danish", value: "da"},
		{ label: "Dutch", value: "nl"},
		{ label: "English", value: "en"},
		{ label: "Estonian", value: "et"},
		{ label: "Filipino", value: "tl"},
		{ label: "Finnish", value: "fi"},
		{ label: "French", value: "fr"},
		{ label: "Galician", value: "gl"},
		{ label: "German", value: "de"},
		{ label: "Greek", value: "el"},
//		{ label: "Hebrew", value: "iw"},
//		{ label: "Hindi", value: "hi"},
		{ label: "Hungarian", value: "hu"},
		{ label: "Indonesian", value: "id"},
		{ label: "Italian", value: "it"},
		{ label: "Japanese", value: "ja"},
		{ label: "Korean", value: "ko"},
		{ label: "Latvian", value: "lv"},
		{ label: "Lithuanian", value: "lt"},
		{ label: "Maltese", value: "mt"},
		{ label: "Norwegian", value: "no"},
//		{ label: "Persian", value: "fa"},
		{ label: "Polish", value: "pl"},
		{ label: "Portuguese", value: "pt-PT"},
		{ label: "Romanian", value: "ro"},
		{ label: "Russian", value: "ru"},
		{ label: "Serbian", value: "sr"},
		{ label: "Slovak", value: "sk"},
		{ label: "Slovenian", value: "sl"},
		{ label: "Spanish", value: "es"},
		{ label: "Swedish", value: "sv"},
//		{ label: "Tagalog", value: "tl"},
//		{ label: "Thai", value: "th"},
		{ label: "Turkish", value: "tr"},
		{ label: "Ukrainian", value: "uk"},
		{ label: "Vietnamese", value: "vi"}
	];
}
TranslateAssistant.prototype.switchLanguages = function(event) {
	var old = this.languageModel.from;
	this.languageModel.from = this.languageModel.to;
	this.languageModel.to = old;
	old = this.textModel.text;
	this.textModel.text = this.textModel.result;
	this.textModel.result = old;
	this.controller.modelChanged(this.languageModel, this);
	this.controller.modelChanged(this.textModel, this);
}

TranslateAssistant.prototype.clearInput = function(event) {
	this.textModel.text = "";
	this.textModel.result = "";
	this.controller.modelChanged(this.textModel, this);
}

TranslateAssistant.prototype.doTranslate = function(event) {
	if (this.textModel.text.length == 0) {
		this.controller.get("translate").mojo.deactivate();
		return;
	}
	this.translateButtonModel.disabled = true;
	this.controller.modelChanged(this.translateButtonModel, this);

	var q = encodeURIComponent(this.textModel.text.toLowerCase());
	var f = this.languageModel.from;
	var t = this.languageModel.to;
	var translateUrl = "http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&q=" + q + "&langpair=" + f + "|" + t;

	new Ajax.Request(translateUrl, {
		method: "get",
		referer: "http://www.u-mass.de",
		evalJSON: "force",
		onSuccess: this.translationSuccess.bind(this),
		onFailure: this.ajaxFailure.bind(this)
	});
}

TranslateAssistant.prototype.sendText = function() {
	this.controller.serviceRequest('palm://com.palm.applicationManager', {
		method: 'launch',
		parameters: {
			id: 'com.palm.app.messaging',
			params: {
				messageText: this.textModel.result
			}
		}
	});
}

TranslateAssistant.prototype.ajaxFailure = function(data) {
	this.translateButtonModel.disabled = false;
	this.controller.modelChanged(this.translateButtonModel, this);
	this.controller.get("translate").mojo.deactivate();
	Mojo.Controller.errorDialog("Could not translate the text.");
}

TranslateAssistant.prototype.translationSuccess = function(data) {
	data = data.responseJSON;
	var text = data.responseData.translatedText;
	text = text.replace(/&#39;/g, "'");
	this.textModel.result = text;
	$("hiddenResult").value = text;
	this.controller.modelChanged(this.textModel, this);
	this.translateButtonModel.disabled = false;
	this.controller.modelChanged(this.translateButtonModel, this);
	this.controller.get("translate").mojo.deactivate();
}

TranslateAssistant.prototype.deactivate = function() {
	cookie.put({
		from: this.languageModel.from,
		to: this.languageModel.to
	});
}